// $Id: wmmixer.cc,v 1.21 2003/03/11 19:40:42 gordon Exp $	
//
// Copyright (C) 2002 Gordon Fraser <gordon@debian.org>
// Parts copyright (C) 1998  Sam Hawker <shawkie@geocities.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// See the COPYING file for more details
//

#include "wmmixer.h"

//--------------------------------------------------------------------
WMMixer::WMMixer(int argc, char** argv)
  : display_(""), position_(""), theme_(DEFAULT_THEME),
    mixer_device_(""), x_flags_(0), wheel_scroll_(DEFAULT_MOUSE_SCROLL)
{
  parseArguments(argc, argv);
  
  try
  {
#ifdef __USE_ALSA__
    if(mixer_device_ != "")
      mixer_ = new AlsaMixer(mixer_device_);
    else
      mixer_ = new AlsaMixer();
#endif
    
#ifdef __USE_OSS__
    if(mixer_device_ != "")
      mixer_ = new OSSMixer(mixer_device_);
    else
      mixer_ = new OSSMixer();
#endif
    
    parseConfigurationFile();
    parseOldConfigurationFile();
    
    dockapp_ = new DockApp(mixer_, argc, argv, display_, position_,
			   theme_, x_flags_);
    event_handler_ = new EventHandler(dockapp_, mixer_, wheel_scroll_);
    
  }
  catch(MixerDeviceException& e)
  {
    std::cerr << e.getErrorMessage() << std::endl;
    exit(1);
  }
  catch(Exception& e)
  {
    std::cerr << e.getErrorMessage() << std::endl;
    exit(1);
  }
  catch(...)
  {
    std::cerr << "Unknown exception caught" << std::endl;
    exit(1);
  }
}


//--------------------------------------------------------------------
WMMixer::~WMMixer()
{
  delete dockapp_;
  delete event_handler_;
  delete mixer_;
}


//--------------------------------------------------------------------
void WMMixer::loop()
{
  event_handler_->loop();
}

//--------------------------------------------------------------------
void WMMixer::parseArguments(int argc, char **argv)
{
  static struct option long_opts[] = {
    {"help",       0, NULL, 'h'},
    {"version",    0, NULL, 'v'},
    {"display",    1, NULL, 'd'},
    {"geometry",   1, NULL, 'g'},
    {"withdrawn",  0, NULL, 'w'},
    {"windowed",   0, NULL, 'W'},
    {"afterstep",  0, NULL, 'a'},
    {"shaped",     0, NULL, 's'},
    {"mix-device", 1, NULL, 'm'},
    {"scrollwheel",1, NULL, 'r'},
    {NULL,         0, NULL, 0  }};
  int i, opt_index = 0;
  

  // For backward compatibility
  for(i=1; i<argc; i++) 
  {
    if(strcmp("-position", argv[i]) == 0) 
    {
      sprintf(argv[i], "%s", "-g");
    } 
    else if(strcmp("-help", argv[i]) == 0) 
    {
      sprintf(argv[i], "%s", "-h");
    } 
    else if(strcmp("-display", argv[i]) == 0) 
    {
      sprintf(argv[i], "%s", "-d");
    }
  }
  
  while ((i = getopt_long(argc, argv, "hvd:g:wWasm:r:", long_opts, &opt_index)) != -1) 
  {
    switch (i) 
    {
    case 'h':
    case ':':
    case '?':
      displayUsage(argv[0]);
      break;
    case 'v':
      displayVersion();
      break;
    case 'd':
      display_ = strdup(optarg);
      break;
    case 'g':
      position_ = strdup(optarg);
      break;
    case 'w':
      x_flags_ |= XHandler::WMAKER;
      break;
    case 'W':
      x_flags_ |= XHandler::WINDOWED;
      break;
    case 'a':
      x_flags_ |= XHandler::ASTEP;
      break;
    case 's':
      x_flags_ |= XHandler::USHAPED;
      break;
    case 'm':
      mixer_device_ = strdup(optarg);
      break;
    case 'r':
      if(atoi(optarg)>0)
	wheel_scroll_ = atoi(optarg);
      break;
    }
  }
}
//--------------------------------------------------------------------
void WMMixer::parseConfigurationFile()
{
  ConfigReader *config_reader = NULL;

  if(getenv("HOME") == 0)
  {
    std::cerr << "$HOME not set, not searching for .wmmixerrc" << std::endl;
    return;
  }

  string filename = getenv("HOME");
  filename += "/.wmmixerrc";
  try
  {
    config_reader = new ConfigReader(filename);
  }
  catch(FileNotFoundException& e)
  {
    return;
  }
  
  char* temp = (char*)config_reader->getStringParameter("theme", "file");  
  if(temp != NULL)
  {
    //    cout << "Theme found: " << temp << endl;
    theme_ = temp;
  }
  else
  {
    // Check if user config file is a valid theme file
    try
    {
      Theme t(config_reader);
      theme_ = filename;
    }
    catch(InvalidThemeException& e)
    {
      theme_ = DEFAULT_THEME;
      // no it's not.
      // leave default.
    }
  }

  int max_channel = 0;
  int channel = 0;

  try
  {
    list<int> channels = config_reader->getIntListParameter("channels", "display");
    if(!channels.empty())
    {
      for(unsigned counter = 0; counter < mixer_->getNumChannels(); counter++)
      {
	mixer_->disableChannel(counter);
      }
      
      for (list<int>::const_iterator i = channels.begin(); i != channels.end(); ++i)
      {
	channel = *i;
	if(channel>=0 && channel< (signed int)mixer_->getNumChannels())
	{
	  mixer_->enableChannel(channel);
	  if(channel>max_channel)
	    max_channel = channel;
	}
      } 
    }
  }
  catch(...)
  {
    // FIXXXME
  }
  
  //  mixer_->setNumChannels(max_channel+1);

  int value = config_reader->getIntParameter("channels", "record_source");  
  if(value)
  {
    mixer_->toggleChannelRecord(value);
  }
  
  for(int current=0; current<max_channel; current++)
  {
    char name[24];
    
    sprintf(name, "volume%d", current);
    value = config_reader->getIntParameter("channels", name);  
    if(value)
    {
      mixer_->setChannelLeft(current, value);
      mixer_->setChannelRight(current, value);
    }
    
    sprintf(name, "volume_left%d", current);
    value = config_reader->getIntParameter("channels", name);  
    if(value)
    {
      mixer_->setChannelLeft(current, value);
    }
    
    sprintf(name, "volume_right%d", current);
    value = config_reader->getIntParameter("channels", name);  
    if(value)
    {
      mixer_->setChannelRight(current, value);
    }
  }
}


//--------------------------------------------------------------------
void WMMixer::parseOldConfigurationFile()
{
  FILE *rcfile;
  char rcfilen[256];
  char buf[256];
  int done;
  unsigned current = 0;
  //   unsigned current = mixer_->getNumChannels() + 1;

  sprintf(rcfilen, "%s/.wmmixer", getenv("HOME"));
  if((rcfile=fopen(rcfilen, "r"))!=NULL)
  {
  
    std::cerr << "Warning: it is recommended to upgrade your configuration file." << std::endl;

    for(unsigned counter = 0; counter < mixer_->getNumChannels(); counter++)
    {
      mixer_->disableChannel(counter);
    }
      
    do
    {
      fgets(buf, 250, rcfile);
      if((done = feof(rcfile)) == 0)
      {
	buf[strlen(buf)-1]=0;
	if(strncmp(buf, "addchannel ", strlen("addchannel "))==0)
	{
	  sscanf(buf, "addchannel %u", &current);
	  if(current >= mixer_->getNumChannels() || 
	     mixer_->isSupported(current) == false)
	  {
	    fprintf(stderr,"%s : Sorry, this channel (%u) is not supported.\n", NAME, current);
	    current = mixer_->getNumChannels() + 1;
	  }
	  else
	  {
	    // 		      channel_list_[num_channels_] = current;
	    // 		      num_channels_++;
	    mixer_->enableChannel(current);
	  }
	}
	if(strncmp(buf, "setchannel ", strlen("setchannel "))==0)
	{
	  sscanf(buf, "setchannel %u", &current);
	  if(current >= mixer_->getNumChannels() || mixer_->isSupported(current)==false)
	  {
	    fprintf(stderr,"%s : Sorry, this channel (%u) is not supported.\n", NAME, current);
	    current = mixer_->getNumChannels() + 1;
	  }
	}
	if(strncmp(buf, "setmono ", strlen("setmono "))==0)
	{
	  if(current== mixer_->getNumChannels() + 1)
	    fprintf(stderr,"%s : Sorry, no current channel.\n", NAME);
	  else
	  {
	    int value;
	    sscanf(buf, "setmono %i", &value);
	    mixer_->setChannelLeft(current, value);
	    mixer_->setChannelRight(current, value);
	  }
	}
	if(strncmp(buf, "setleft ", strlen("setleft "))==0)
	{
	  if(current== mixer_->getNumChannels() + 1)
	    fprintf(stderr, "%s : Sorry, no current channel.\n", NAME);
	  else
	  {
	    int value;
	    sscanf(buf, "setleft %i", &value);
	    mixer_->setChannelLeft(current, value);
	  }
	}
	if(strncmp(buf, "setright ", strlen("setright "))==0)
	{
	  if(current== mixer_->getNumChannels() + 1)
	    fprintf(stderr, "%s : Sorry, no current channel.\n", NAME);
	  else
	  {
	    int value;
	    sscanf(buf, "setleft %i", &value);
	    mixer_->setChannelRight(current, value);
	  }
	}
	if(strncmp(buf, "setrecsrc ", strlen("setrecsrc "))==0)
	{
	  if(current== mixer_->getNumChannels() + 1)
	    fprintf(stderr, "%s : Sorry, no current channel.\n", NAME);
	  else
	    mixer_->toggleChannelRecord(current);
	  //		    mixer_->setRec(current, (strncmp(buf+strlen("setrecsrc "), "true", strlen("true"))==0));
	}
      }
    }  
    while(done==0);
    fclose(rcfile);
    //       mixer_->writeRec();

  }
}

//--------------------------------------------------------------------
void WMMixer::displayUsage(const char* name) const
{
  cout << "Usage: " << name << "[options]" << endl;
  cout << "  -h,  --help                    display this help screen" << endl;
  cout << "  -v,  --version                 display program version" << endl;
  cout << "  -d,  --display <string>        display to use (see X manual pages)" << endl;
  cout << "  -g,  --geometry +XPOS+YPOS     geometry to use (see X manual pages)" << endl;
  cout << "  -w,  --withdrawn               run the application in withdrawn mode" << endl;
  cout << "                                 (obsolete, is default now)" << endl;
  cout << "  -W,  --windowed                run the application in windowed mode" << endl;
  cout << "                                 (for WindowMaker, etc)" << endl;
  cout << "  -a,  --afterstep               use smaller window (for AfterStep Wharf)" << endl;
  cout << "  -s,  --shaped                  shaped window" << endl;
#ifdef __USE_OSS__
  cout << "  -m,  --mix-device              use specified device (rather than /dev/mixer)" << endl;
#endif
#ifdef __USE_ALSA__
  cout << "  -m,  --mix-device              use specified device (rather than /dev/mixer)" << endl;
#endif
  cout << "  -r,  --scrollwheel <number>    volume increase/decrease with mouse wheel (default: 2)" << endl;
  cout << "\nFor backward compatibility the following obsolete options are still supported:" << endl;
  cout << "  -help                          display this help screen" << endl;
  cout << "  -position                      geometry to use (see X manual pages)" << endl;
  cout << "  -display                       display to use (see X manual pages)" << endl;
  exit(0);

}

//--------------------------------------------------------------------
void WMMixer::displayVersion() const
{
  cout << "wmmixer version 2.0beta4" << endl;
  exit(0);
}


//====================================================================
int main(int argc, char** argv)
{
  WMMixer wmmixer = WMMixer(argc, argv);
  wmmixer.loop();
  exit(0);
}
